package geniusweb.profile.utilityspace;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.mock;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import org.junit.Test;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.issuevalue.DiscreteValue;
import geniusweb.issuevalue.Value;
import tudelft.utilities.junit.GeneralTests;

public class ProductOfValueTest extends GeneralTests<ProductOfValue> {
	private final static ObjectMapper jackson = new ObjectMapper();

	private final static Value value1 = new DiscreteValue("val1");
	private final static Value value2 = new DiscreteValue("val2");
	private final static ProductOfValue values1 = new ProductOfValue(
			Arrays.asList(value1, value2));
	private final static ProductOfValue values1a = new ProductOfValue(
			Arrays.asList(value1, value2));
	private final static ProductOfValue values2 = new ProductOfValue(
			Arrays.asList(value2));
	private final static DiscreteValue low = new DiscreteValue("low");
	private final static DiscreteValue high = new DiscreteValue("high");

	private String jsontext = "[\"val1\",\"val2\"]";

	@Override
	public List<List<ProductOfValue>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(values1, values1a),
				Arrays.asList(values2));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList(".*\"val1\", \"val2\".*", ".*\"val2\".*");
	}

	@Test
	public void testSerialize()
			throws JsonParseException, JsonMappingException, IOException {

		assertEquals(jsontext, jackson.writeValueAsString(values1));
	}

	@Test
	public void testBasicDeserialize()
			throws JsonParseException, JsonMappingException, IOException {
		System.out.println(jsontext);
		assertEquals(values1,
				jackson.readValue(jsontext, ProductOfValue.class));
	}

	@Test
	public void testMerge() {
		Value val1 = mock(Value.class);
		Value val2 = mock(Value.class);
		Value val3 = mock(Value.class);
		Value val4 = mock(Value.class);

		ProductOfValue prodOfVal1 = new ProductOfValue(
				Arrays.asList(val1, val2));
		ProductOfValue prodOfVal2 = new ProductOfValue(
				Arrays.asList(val3, val4));

		ProductOfValue prodVal = prodOfVal1.merge(prodOfVal2);
		assertEquals(4, prodVal.getValues().size());
	}

}
